<?PHP
/**
 * Start page of the backend.
 * Offers the welcome message (if there is one),
 * a description of the CMS and some basic data
 * on the contents of the current page.
 *
 * @author Joshua Ramon Enslin <joshua@jrenslin.de>
 */

/*
 * Require files and ensure environment.
 */

require_once __DIR__ . "/inc/functions.php";

ensureEnvironment();                                    // Ensure existence of system files.
$translations = loadLanguage($settings['defaultLang']); // Load translations.
ensureBackendEnv();                                     // Ensure session is started etc.
$pages = loadPages();                                   // Load overview of pages.

/*
 * Load data.
 */

/*
 * Output
 */

echo printBackendHead($settings, $translations['start'], $translations['start'], $settings['logo']);
echo printBackendHeader($translations['start'], $translations['helpStart']);

echo '
<div id="mainWrapper">
';

echo printBackendNav($translations);

echo '
    <main>';

echo printStaticPagePart("welcomeMsg", "section"); // Print aside (if need be)

echo '
    <section class="startPgTiles">

        <div>
            <h3>' . $translations['aboutThisSite'] . '</h3>
            <table>
                <tr>
                    <th>' . $translations['numberOfPages'] . '</th>
                    <td>' . (string)count($pages) . '</td>
                </tr>
                <tr>
                    <th>' . $translations['numberOfPublicPages'] . '</th>
                    <td>' . (string)count(loadPublicPages()) . '</td>
                </tr>
                <tr>
                    <th>' . $translations['phpVersion'] . '</th>
                    <td>PHP ' . (string)phpversion() . '</td>
                </tr>
            </table>
        </div>

        <div>
            <h3>' . $translations['aboutMDCMS'] . '</h3>
            ' . $translations['aboutMDCMS_content'] . '
        </div>

    </section>
';

echo '
    </main>
</div>';

echo printBackendEnd();

?>
